/*	$OpenBSD: mktemp.c,v 1.4 1997/06/20 04:17:42 millert Exp $	*/

/*
 * Copyright (c) 1996 Todd C. Miller <Todd.Miller@courtesan.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Todd C. Miller.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef lint                                                              
static char rcsid[] = "$OpenBSD: mktemp.c,v 1.4 1997/06/20 04:17:42 millert Exp $";
#endif /* not lint */                                                        

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <stdarg.h>

extern char *__progname;

void err(int somecode, char * message, ...) {
    va_list arglist;

    va_start(arglist, message);
    vfprintf(stderr, message, arglist);
    fprintf(stderr, "\n");
    va_end(arglist);

    exit(somecode);
}

void errx(int somecode, char * message) {
    err(somecode, message);
}

void
usage()
{
	(void) fprintf(stderr, "Usage: %s [-d] [-q] [-u] template\n",
	    __progname);
	exit(1);
}



/* 
    glibc 2.1 compat-patch,
    needed for updating RH 6.x (EE) - systems to kernel 2.4,
    mkdtemp() was introduced with glibc 2.2
*/
#include <features.h>
#if ( __GLIBC__ < 2 ) || ( __GLIBC__ == 2  && __GLIBC_MINOR__ < 2 )

// #include <sys/stat.h>
// #include <sys/types.h>
// #include <fcntl.h>
// #include <unistd.h>    

static char * mkdtemp (char *template)
{
    char temp[strlen(template) + 1];
    int ret;

    while (1) {
        strcpy(temp, template);
        if (mktemp(temp) == NULL) {
            return NULL;
        }
        ret = mkdir(temp, 0700);
        /* success, return the name of the new directory */
        if (ret == 0) {
            strcpy(template, temp);
            return template;
        }
        /* failed because directory existed already -- just try again */
        if ((ret == -1) && (errno == EEXIST))
            continue;
        /* failed for any other reason, return the name of the directory we tried to create */
        strcpy(template, temp);
        return NULL;
    }
}

#endif




int
main(argc, argv)
	int argc;
	char **argv;
{
	char *template;
	int c, uflag = 0, qflag = 0, makedir = 0;

	while ((c = getopt(argc, argv, "dqu")) != -1)
		switch(c) {
		case 'd':
			makedir = 1;
			break;
		case 'q':
			qflag = 1;
			break;
		case 'u':
			uflag = 1;
			break;
		case '?':
		default:
			usage();
	}

	if (argc - optind != 1)
		usage();

	if ((template = strdup(argv[optind])) == NULL) {
		if (qflag)
			exit(1);
		else
			errx(1, "Cannot allocate memory");
	}

	if (makedir) {
		if (mkdtemp(template) == NULL) {
			if (qflag)
				exit(1);
			else
				err(1, "Cannot make temp dir %s", template);
		}

		if (uflag)
			(void) rmdir(template);
	} else {
		if (mkstemp(template) < 0) {
			if (qflag)
				exit(1);
			else
				err(1, "Cannot create temp file %s", template);
		}

		if (uflag)
			(void) unlink(template);
	}

	(void) puts(template);
	free(template);

	exit(0);
}
